# -*- coding: utf-8 -*-
""" scaffy package. """
from __future__ import absolute_import, unicode_literals

# stdlib imports
from os.path import abspath, dirname


__version__ = '0.8.1'
PKG_DIR = abspath(dirname(__file__))
